/*
BASE FRAMEWORK FOR CHIRP (Codename "L")
CHIRP is (c) 2008 Warm Chocolate Widgets.

BaseFramework.js taken from the Macworld UK Widget (prelude.js) by The Mac Place (www.themacplace.co.uk).

This file contains functions for the framework of the widget, which are used in the engine (ChirpEngine.js) and the main widget HTML file (Chirp.html).

YOU DON'T NEED TO CHANGE ANYTHING HERE!
*/

//BROWSER: Sets variables for browsers, and provides browser-specific instructions.
var IsDashboard;
var IsMozilla;
var IsIE;

function init_prelude(noTabs)
{
    IsDashboard = (window.widget ? true : false);
    
    try {
        IsMozilla = (netscape.security.PrivilegeManager ? true : false);
    } catch (e) {
        IsMozilla = false;
    }
    
    IsIE = (window.ActiveXObject ? true : false);
    
    if (IsDashboard)
    {
        widget.onhide = on_hide;
        widget.onshow = on_show;
    }
    
    if (!noTabs)
    {
		flipperInit();
		tabsInit();
	}
}


//Show or hide functions--when I show or hide Dashboard, what should I do?
function on_show()
{
    rss_refresh();
	timer_resume();
}

function on_hide()
{
    timer_stop();
}


/* PREFERENCES FRAMEWORK

Framework for creating preference keys and cookies (Dashboard writes to prefrence keys in the .plist, Safari creates temporary cookies for testing). Portions of this script are taken from http://www.quirksmode.org/js/cookies.html, and adapted for .plist writing functions
*/

//Create a new cookie for a prefrence (Safari and Mozilla)
function create_cookie(name,value)
{
	var date = new Date();
	date.setFullYear(3000);
	document.cookie = name+"="+value+"; expires="+date.toGMTString()+"; path=/";
}

//Get the preference information from a cookie (Safari and Mozilla)
function read_cookie(name)
{
	var nameEQ = name + "=";
	var ca = document.cookie.split(';');
	for(var i=0;i < ca.length;i++)
	{
		var c = ca[i];
		while (c.charAt(0)==' ') c = c.substring(1,c.length);
		if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
	}
	return undefined;
}

//Access the information stored in a .plist key (Dashboard), or access the information stored in a temporary cookie (Safari and Mozilla)
function get_preference(key, def)
{
	var res;
	if (IsDashboard)
		res = widget.preferenceForKey(key)
	else
		res = read_cookie(key);
	
	if (res)
		return res;
	else
		return def;
}

//Create a .plist key, or write changes to one (Dashboard), or create a temporary cookie and write changes to it (Safari and Mozilla)
function set_preference(key, value)
{
	if (IsDashboard)
		widget.setPreferenceForKey(value, key);
	else
		create_cookie(key, value);
}


/*DOWNLOADING THE XML FEED

Uses XMLHttpRequest to download the RSS feed.
*/

//Gives each download a unique identifier
//There is a bug in Dashboard (http://www.dashboardwidgets.com/forums/viewtopic.php?t=235) that causes Dashboard to cache each download. The first time, the download is successful, but the following times, the download fails by returning an error ("undefined") instead of status 200 (OK). These functions create a unique ID for each download, so that Dashboard is able to work around this.
var download_uniqueid = 1;

function debug_http(msg)
{
    //These are the debug functions, leave them alone
	//debug_line(msg);
}

function download_async_text(callback, url)
{
	download_async_any(callback, url, false);
}

function download_async_xml(callback, url)
{
	download_async_any(callback, url, true);
}


function download_async_any(callback, url, isXML)
{
    //debug_http("DLD:" + url + "&uniqueid=" + download_uniqueid);

    var Req;
    if (IsIE)
        Req = new ActiveXObject("Microsoft.XMLHTTP");
    else
        Req = new XMLHttpRequest();
    
	if (IsMozilla)
        netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");

    Req.onreadystatechange = mycallback;
    download_uniqueid++;
    Req.open("GET", url +
    	(url.indexOf("?") == -1 ? "?" : "&") + "uniqueid=" + download_uniqueid, true);
    Req.send(null);

    function mycallback()
    {
        /*
        try {
            debug_http(Req.readyState + ":" + Req.status + ":" + Req.statusText + ":" + url);
        } catch (e) {
            debug_http(Req.readyState + ":_:_:" + url);
        }
        */
        
    
        if (Req.readyState != 4) return;
        
        if (Req.status == 200)
        {
        	if (isXML)
        	{
				if (IsMozilla)
					netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
        		callback(true, Req.responseText, Req.responseXML.documentElement);
        	}
        	else
	            callback(true, Req.responseText);
        }
        else
            callback(false, Req.status + ": " + Req.statusText);
    }
}


/* PANELS/TABS

Functions for showing and hiding the front and back of the widget (configuration and main view). Also has animation function for flipping the widget. The one listed first in the arguments is the default tab (in which the function begins). Including these functions in the Base Framework ensures compatibility with pre-Mac OS X 10.4.3 systems, which do not contain all of the Apple Classes (an alternative way to create the buttons).
*/

function tabsInit()
{
	tabsFireEvent("main");
}

function showConfig()
{
	tabsShow("config", "main", "ToBack");
}

function showMain()
{
	tabsShow("main", "config", "ToFront");
	rss_refresh();//Inserted by WCWidgets--when you flip the widget to the front side, refresh the RSS headlines to reflect the new username and password settings.
}

function tabsShow(show, hide, transname)
{
    if (IsDashboard) widget.prepareForTransition(transname);
    
    tabsFireEvent(show);
    document.getElementById(show).style.display = "block";
    document.getElementById(hide).style.display = "none";

    if (IsDashboard) setTimeout('widget.performTransition();', 0);
}

function tabsFireEvent(id)
{
	var s = document.getElementById(id).getAttribute("onshow");
	if (s) eval(s);
}

function flipperInit()
{
	var mainTab = document.getElementById("main");
	mainTab.setAttribute("onmousemove", "flipperShow()");
	mainTab.setAttribute("onmouseout", "flipperHide()");
	animInit("showConfig", setConfigOpacity, 0, 1, 10);
}

function setConfigOpacity(x)
{
	document.getElementById("showConfig_i").style.opacity = x;
}

function flipperShow()
{
	animRunMax("showConfig");
}

function flipperHide()
{
	animRunMin("showConfig");
}

function btnShowConfig(func)
{
	document.write(
		'<div id="showConfig_i"></div>' +
		'<a href="javascript:' + (func ? func : 'showConfig()') + '" id="showConfig"></a>'
		);
}


/* ANIMATORS

Functions for animation. An animation contains the following: begin, end, steps, value, f, interval. An animation can also have 3 directions: -1 (minimum), 1 (maximum), 0 (nothing changes). Again, Including these functions in the Base Framework ensures compatibility with pre-Mac OS X 10.4.3 systems. Much of this animation function is managed by the AppleAnimator class in Mac OS X 10.4.3 and higher.
*/
var animData = {};

function animInit(name, f, begin, end, steps)
{
	f(begin);
	animData[name] = {f:f, value:begin, begin:begin, end:end, steps:steps, direction: 0};
}

function animRunMax(name)
{
	animRun(name, 1);
}

function animRunMin(name)
{
	animRun(name, -1);
}

function animRun(name, direction)
{
	var dat = animData[name];
	dat.direction = direction;
	
	if (!animFinished(name) && !dat.interval)
		dat.interval = window.setInterval("animFire(\'" + name + "\');", 25);
}

function animFinished(name)
{
	var dat = animData[name];
	if (
		(dat.direction == -1 && dat.value <= dat.begin) ||
		(dat.direction == 01 && dat.value >= dat.end  ) ||
		(dat.direction == 0))
	{
		if (dat.interval)
		{
			window.clearInterval(dat.interval);
			dat.interval = undefined;
		}
		dat.direction = 0;
		return true;
	}
	else
		return false;
}

function animNext(name)
{
	var dat = animData[name];
	var val = dat.value + (((dat.end - dat.begin) / dat.steps) * dat.direction);
	val = Math.min(dat.end, Math.max(dat.begin, val));
	dat.value = val;
	dat.f(val);
}

function animFire(name)
{
	if (!animFinished(name))
		animNext(name);
}

/* BASIC FRAMEWORK FUNCTIONS

Basic functions, used throughout the widget for operation.
*/

function title_case(x)
{
    var res = "";
    var makeLower = false;
    for (var i = 0; i < x.length; i++)
    {
        var c = x.charAt(i);
        var cUpper = (c >= "A" && c <= "Z");
        if (cUpper && makeLower)
            res += c.toLowerCase();
        else
        {
            makeLower = cUpper;
            res += c;
        }
    }
    return res;
}

//Cross-compatible openURL script (If you're in Safari, set the window location to the URL, if you're in Dashboard, use the built-in function to hide the Dashboard and then open the URL in Safari.) Safari functionality is built-in to help you test the widget.
function openURL(x)
{
	if (IsDashboard)
		widget.openURL(x);
	else
		window.open(x);
}

//Function to display dynamic HTML content (used in ChirpEngine.js to display the RSS headlines)
function setHTML(id, html)
{
	var elem = document.getElementById(id);
	if (elem)
		elem.innerHTML = html;
	else
		throw("setHTML: id=" + id);
	
}

function cnum(x)
{
	return x*1;
}


/* TIMERS

Functions  to start, stop, and resume the timers (used for the animation functions). These can also be implemented to refresh the RSS feed (enter the function you want to set a timer for for "callback" and the time in milliseconds for "interval" for function "timer_start"), although this is disabled for Chirp because of authentication problems with the timer.
*/
var IsTimer = false;
var IsTimerPaused;
var TimerCallback;
var TimerInterval;

function timer_start(callback, interval)
{
    IsTimer = true;
    IsTimerPaused = false;
    TimerCallback = callback
    TimerInterval = interval;
    timer_begin();
}

function timer_begin()
{
    window.setTimeout("timer_fire()", TimerInterval);
}

function timer_fire()
{
    if (IsTimer && !IsTimerPaused)
    {
        TimerCallback();
        timer_begin();
    }
}

function timer_stop()
{
    IsTimer = false;
}

function timer_resume()
{
    IsTimerPaused = false;
    timer_fire();
}

function timer_pause()
{
    IsTimerPaused = true;
}


/*DEBUGGING

Function for presenting the debugging output/code, leave this alone.
*/

function debug_line(s)
{
    var elem = document.getElementById("debug");
    elem.value += "\n" + s;
    elem.style.display = "block";
}

/*END OF BaseFramework.js*/