/*

CHIRP (Codename "L") ENGINE, (c) 2008 Warm Chocolate Widgets.

RSS Timeline functions and configuration functions are based on the Macworld UK News Widget by The Mac Place (www.themacplace.co.uk). Scroller functions are adapted from the PhilMUG RSS Widget (www.philmug.ph).

Special thanks to Kris Ziel (www.ampersandsoftware.com) and OpenJS Scripts (www.openjs.com) for providing the XMLHttpRequest scripts, on which the status_tweet function is built on, and to Media College (www.mediacollege.com) for providing the limitText function.

The Chirp Engine uses the Twitter API to post and download your status updates. You can learn about the various Twitter API commands at the official Twitter API documentation web site: http://groups.google.com/group/twitter-development-talk/web/api-documentation.

*/

//PREVENTS INTERFACE GLITCHES, DON'T MODIFY THIS
var ignore_config = true;

/*INITIALIZATION SECTION

Load the base JavaScript framework (BaseFramework.js), load the saved configuration, and display the "getting started" message.
*/

function init()
{
    init_prelude();
   
    load_config();
  
	//What should I display when I welcome you/when you haven't posted anything yet?
	setHTML("results", '<strong><em>To get started, tell us what you\'re doing in the box above!</em></strong>');
	refresh_scroller();
	
}

//Load saved preferences from the .plist, or load the default configuration if no .plist exists
function load_config()
{
	f("background","raspberry");//Default colour scheme; font and font size settings (below) are automatically set by ChirpInterface.css and hidden fields in Chirp.html
	f("font-family","monospace");
	f("font-size","small");
	
	function f(key, def)
	{
		var x = get_preference(key, def);
		document.getElementById(x).checked = true;
	}
	
    config_change();
}

//SAVE COLOUR SCHEME SETTINGS, FONT, AND FONT SIZE (font and font size are defined by hidden fields in Chirp.html) TO THE .PLIST
function config_change()
{
	var items = document.getElementsByTagName("input");
	var cls = "";
	for (var i = 0; i < items.length; i++)
	{
		var item = items[i];
		
		if (item.checked)
		{
			var att = item.getAttribute("id");
			var mod = item.getAttribute("name");
			
			set_preference(mod, att);
			
			//Make sure that a black rollover button is used on white or light backgrounds (the attribute for black text can be set in ChirpInterface.css)
			if (att == "creme")				//Creme Style
				att += " blackConfig";
			if (att == "quicksilver")		//Quicksilver Style
				att += " blackConfig";
			cls += att + " ";
		}
	}
	document.getElementById("main").className = cls;
}

/* FUNCTIONS TO SAVE USERNAME AND PASSWORD TO THE .PLIST

These functions write the user name and password to either a .plist or a cookie, depending on wether you're in Dashboard or Safari (the set_preference function is outlined in BaseFramework.js). These functions are called using the JavaScript "onchange" event handler. Look at Chirp.html (form ID "signin") to see how these functions are implemented.
*/
//Save the user name
function save_plist_username()
{
	if (window.widget){
	//Get the value of the user name or e-mail address from what's entered
	var signin_name = document.getElementById("username").value;
	{
		if (signin_name != "undefined"){//Make sure to save only if there's something entered in the form field
			set_preference("username_or_email",signin_name); //And write the preference
		}
	}
	}
}

//Save the password
function save_plist_password()
{
	if (window.widget){
	var signin_password = document.getElementById("password").value;
	{
	if (signin_password != "undefined"){
		set_preference("password",signin_password);
	}
	}
	}
}

/*FUNCTION TO LIMIT UPDATE TEXT

Script to limit the characters that can be entered in the update field, since Twitter only allows a maximum of 140 characters per post. In addition to limiting the amount of characters that can be entered, this script also allows for a countdown field to show you how many characters you have left. Look at Chirp.html to see how the limit text and countdown functions are implemented.

Script by Media College (http://www.mediacollege.com).*/
function limitText(limitField, limitCount, limitNum) {
	if (limitField.value.length > limitNum) {
		limitField.value = limitField.value.substring(0, limitNum);
	} else {
		limitCount.value = limitNum - limitField.value.length;
	}
}

/*FUNCTION TO POST YOUR STATUS UPDATE TO TWITTER

Script based on Kris Ziel's "XMLHttpRequest" Script (http://www.ampersandsoftware.com) and OpenJS's "Post using XMLHttpRequest" Script (http://www.openjs.com), with major modifications*/

//This function creates a cross-browser compatible XMLHttpRequest
function createXMLHttpRequest() {
    if (typeof XMLHttpRequest != "undefined") {//If you're running Safari, Mozilla/Firefox, Internet Explorer 7.0, Opera or any other browser besides Internet Explorer 5.5-6.0
        return new XMLHttpRequest();//Then create a new XMLHttpRequest using the XMLHttpRequest function
		
    } else if (typeof ActiveXObject != "undefined") {//Else, if you're running Internet Explorer 5.5 or 6.0
        return new ActiveXObject("Microsoft.XMLHTTP");//Then create a new XMLHttpRequest using Microsoft's AJAX XMLHTTP function
    }
}
function status_tweet()
{
	if (window.widget){//If you're running Dashboard
	var my_user_id = widget.preferenceForKey("username_or_email");//Get the username from the .plist
	var my_password = widget.preferenceForKey("password");//Get the password from the .plist
	}
	else{//Since Safari can't access the widget's .plist, you want it to get the username and password directly from what you entered on the configuration side
	var my_user_id = document.getElementById("username").value;//Get the username from the configuration side
	var my_password = document.getElementById("password").value;//Get the password from the configuration side
}
	//Create a new cross-browser XMLHttpRequest (using the "createXMLHttpRequest" function) and name it "xmlhttp"
	var xmlhttp = createXMLHttpRequest();
	//Get the text from this textarea in the widget
	var status_doc = document.getElementById("post_this").value;
	//The URL to which you want to post your status update (username:password@domain.com)
	var twitter_submit = "http://" + my_user_id + ":" + my_password + "@twitter.com/statuses/update.xml";
	//Attach the identifier ("status=") as well as your status message (identified by the variable). The identifier is rwquired, otherwise, Twitter won't know what it is you're posting.
	var status = "status=" + status_doc;
	//Open a connection to the Twitter URL using the POST method (you can only make authenticated POST requests to Twitter
	xmlhttp.open("POST", twitter_submit, true);

//Send required posting header before sending the actual status
xmlhttp.setRequestHeader("Cache-Control", "no-cache");//A bug in Dashboard causes an error on posting if caching is enabled.
xmlhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
xmlhttp.setRequestHeader("Content-length", status_doc.length);//Length of your status is required. Since the status update textarea prevents you from entering more than 140 characters, this value will automatically be 140 or less, depending on your status update's length. If it's somehow more than 140 characters, Twitter will automatically truncate it.
xmlhttp.setRequestHeader("Connection", "close");

xmlhttp.onreadystatechange = function() {//What should be done when the Twitter server returns an HTTP code.
	//If the code is 200 (everything is OK and the Twitter server recieved your request), what should be done next?
	if(xmlhttp.readyState == 4 && xmlhttp.status == 200) {
		//Clear the form so that you can post a new status update
		document.post.reset();
		//And refresh the RSS feed for your recent updates, containing your latest status update
		rss_refresh();
		
		/* DEBUGGING THE SCRIPT
		
		If, for some reason, your status isn't posted, you'll want to comment out the rss_refresh function above, and enable the function below. This will return the XML response from the Twitter server in an alert JavaScript box. In that box, pay particular attention to the code between the <text> characters. This contains your status (or what the Twitter server received as your status. If you have any problems, or the Twitter server refuses to accept your request, enable this option to find out what the Twitter server is getting from you.*/
		
		//alert(xmlhttp.responseText);
	}
}
//ASSEMBLE THE URL AND SEND YOUR STATUS UPDATE
xmlhttp.send(status);
}

//FUNCTION TO REFRESH THE RSS FEED
function rss_refresh()
{
 //This is the RSS feed for your friends' timeline (containing you and your friends' last 20 updates). You can change this to the public timeline or user timeline (or any other feed, for that matter). Use the Twitter APi documentation to find out the addresses for the timelines (link is at the beginning of this document). Note that, since the friends' timeline requires authentication, it needs to have the username or email and password embedded in the address.
if (window.widget){
	var rss_user_id = widget.preferenceForKey("username_or_email");//Get the username from the .plist
	var rss_password = widget.preferenceForKey("password");//Get the password from the .plist
}else{
	var rss_user_id = document.getElementById("username").value;//Get the username from the configuration side
	var rss_password = document.getElementById("password").value;//Get the password from the configuration side
}
	download_async_xml(rss_display, "http://" + rss_user_id + ":" + rss_password + "@twitter.com/statuses/friends_timeline.rss");//And assemble the URL
}
	refresh_scroller();//And finally, refresh the scroller to account for the new content

//FUNCTION TO PARSE THE DOWNLOADED RSS FEED
function rss_display(success, text, xml)
{
    //If I couldn't download the timeline (because you're not signed in or connected to the Internet, you have exceeded your requests per hour, or if Twitter is down)...
	if (!success)
    {
    	//Then display an error message. Do NOT modify the "results" portion, this instructs the engine to display the error in the "results" div (see Chirp.html).
		setHTML("results", "<strong><em>Sorry! There was a problem loading your latest updates. This may be because you have exceeded your hourly refresh rate, or because Twitter is down. Please try again in a few minutes...</em></strong>");
        return;
    }
    
    //Functions to get the tags from the XML code
	var s = "";
    var items = xml.getElementsByTagName("item");
    
    for (var i = 0; i < items.length; i++)
    {
    	var item = items[i];
    	
    	var vals = {};
    	for (var j = 0; j < item.childNodes.length; j++)
    	{
    		var jj = item.childNodes[j];
    		vals[jj.nodeName] = txt(jj);
    	}
    	
    	/*HOW SHOULD I DISPLAY EACH ENTRY?
		
		The strings inside the "vals" function are variables/tags that are fetched from the XML/RSS feed (some examples include: "title" (the title of the item), "link" (the HTML link to the entry), "description" (an explanation of the title; in Twitter, the description is the same as the title so, to avoid redundancy, the description is not entered here), and "pubDate" (the date and time the entry was published. Be careful with this one, as the pubDate is often displayed in RFC 822 format in UTC or GMT time. It can be confusing to many users, which is why you should take care when using this string).
																																																																																													
		Including the "openURL" function here allows you to open the Twitter page containing your status update when you click on the link. The variable "s" is assigned to each item.*/																																																																																																																																																																																
		s += '<div class="item">' +
    			'<a onclick="openURL(\'' + vals["link"] + '\')">' + vals["title"] + '</a>' +
    		 '</div>'
   	}   	

		//Display each of the RSS items (defined by the variable "s") in the "results" div.
		setHTML("results", s);
		
		//And refresh the scrollbar
		refresh_scroller();
}

//Parses the downloaded XML to make sure that it's an XML tag and not an empty space. This is required due to a rendering error that occurs in Netscape browsers (see http://www.quirksmode.org/dom/importxml.html for more information). Don't change this.
function txt(x)
{
	if (x && x.firstChild)
		return x.firstChild.nodeValue;
	else
		return x;
}

/*FUNCTION TO INITIALIZE AND DISPLAY THE SCROLLBAR

Taken from the PhilMUG RSS Widget (www.philmug.ph). This function initializes the scrollbar (for the content that you want to scroll) and displays it according to the CSS elements outlined in ScrollbarInterface.js.
*/
function refresh_scroller() {
	//Use the divs from Chirp.html (the divs are empty, but they get their information from the CSS file) to initialize the scroller and to outline where it should be displayed.
	scrollerInit(document.getElementById("myScrollBar"), document.getElementById("myScrollTrack"), document.getElementById("myScrollThumb"));
	//What should I scroll? (Replace "results" with the div identifier containing the content that you want to scroll. For this widget, since the RSS headlines are displayed in the "results" div, this is what we want to scroll.)
	calculateAndShowThumb(document.getElementById("results"));

	if (!window.widget)
	{
		show ();
	}
}

/*END OF ChirpEngine.js*/